clc;clear all;close all;

addpath([cd,'\..\nonstationaryMultichanNoiseGenerator']);
addpath([cd,'\..\functions\']);

setup.sensorDistance = 0.1;
setup.speedOfSound = 343;
setup.nSensors = 3;
setup.noiseField = 'spherical';
setup.reverbTime = 0.2;

setup.sdnr = 0;
setup.ssnr = 30;

setup.nWin = 60;
setup.forgetNoi = 0.8;
setup.forgetSig = 0.8; 

setup.filtStrings = {'wiener','mvdr','maxSnr','trOff'};
setup.trOff.signalRanks = setup.nWin;
setup.trOff.muGrid = 0:0.2:1.2;

display(['Running script: ',mfilename]);
display(' ');

display('Enhancing...');
for idx = 1:length(setup.trOff.muGrid),
    setup.trOff.mu = setup.trOff.muGrid(idx);
    [signals,setup] = multichannelSignalGenerator(setup);

    [simulationData,setup] = vsTimeDomEnhanceMultChanSignals(signals,setup);

    performance(idx,1) = vsTimeDomMultichannelMeasurePerformance(simulationData,setup,1);
end

%%
display('Measuring performance...');
for idx = 1:length(setup.trOff.muGrid),
    iSnrMean(1,idx) = performance(idx,1).noiseReduction.iSnr.mean;
    oSnrMaxSnrMean(1,idx) = performance(idx,1).noiseReduction.oSnr.maxSnr.mean;
    oSnrWienerMean(1,idx) = performance(idx,1).noiseReduction.oSnr.wiener.mean;
    oSnrMvdrMean(1,idx) = performance(idx,1).noiseReduction.oSnr.mvdr.mean;
    oSnrTrOffMean(1,idx) = performance(idx,1).noiseReduction.oSnr.trOff.mean;

    dsdMaxSnrMean(1,idx) = performance(idx,1).signalDistortion.dsd.maxSnr.mean;
    dsdWienerMean(1,idx) = performance(idx,1).signalDistortion.dsd.wiener.mean;
    dsdMvdrMean(1,idx) = performance(idx,1).signalDistortion.dsd.mvdr.mean;
    dsdTrOffMean(1,idx) = performance(idx,1).signalDistortion.dsd.trOff.mean;
end

%% save
% dateString = datestr(now,30);
% 
% save([mfilename,'_',dateString,'.mat']);

%% plot
close all;
figure(1);
plot(10*log10(mean(iSnrMean,3)),'k');
hold on;
plot(10*log10(mean(oSnrMaxSnrMean,3)));
plot(10*log10(mean(oSnrWienerMean,3)));
plot(10*log10(mean(oSnrMvdrMean,3).'),'g');
plot(10*log10(mean(oSnrTrOffMean,3).'),'m');
hold off;

figure(2);
plot(10*log10(mean(dsdMaxSnrMean,3)));
hold on;
plot(10*log10(mean(dsdWienerMean,3)));
plot(10*log10(mean(dsdMvdrMean,3).'),'g');
plot(10*log10(mean(dsdTrOffMean,3).'),'m');
hold off;
