function [performance] = vsTimeDomMultichannelMeasurePerformance(data,setup,flagFromSignals)

nBlockSkip = 10;

if flagFromSignals,
    
    % raw signal powers
    [performance.power.raw.sigPow,performance.power.raw.sigPowMean] = ...
    calculatePowers(data.raw.sigFrames,nBlockSkip);

    % raw noise powers
    [performance.power.raw.noiPow,performance.power.raw.noiPowMean] = ...
    calculatePowers(data.raw.noiFrames,nBlockSkip);
    
    [performance.noiseReduction.iSnr.overTime,performance.noiseReduction.iSnr.mean] ...
        = measurePerformance(performance,'raw');
    
    for iFiltStr=1:length(setup.filtStrings),
        switch char(setup.filtStrings(iFiltStr)),
            case 'maxSnr',
            % signal and noise powers (max snr)   
            [performance.power.maxSnr.sigPow,performance.power.maxSnr.sigPowMean] = ...
                calculatePowers(data.maxSnr.sigFrames,nBlockSkip);
            
            [performance.power.maxSnr.noiPow,performance.power.maxSnr.noiPowMean] = ...
                calculatePowers(data.maxSnr.noiFrames,nBlockSkip);
            
            [performance.noiseReduction.oSnr.maxSnr.overTime,...
                performance.noiseReduction.oSnr.maxSnr.mean,...
                performance.signalDistortion.dsd.maxSnr.overTime,...
                performance.signalDistortion.dsd.maxSnr.mean] ...
                = measurePerformance(performance,char(setup.filtStrings(iFiltStr)));
            case 'wiener',
            % signal and noise powers (wiener)   
            [performance.power.wiener.sigPow,performance.power.wiener.sigPowMean] = ...
                calculatePowers(data.wiener.sigFrames,nBlockSkip);
            
            [performance.power.wiener.noiPow,performance.power.wiener.noiPowMean] = ...
                calculatePowers(data.wiener.noiFrames,nBlockSkip);
            
            [performance.noiseReduction.oSnr.wiener.overTime,...
                performance.noiseReduction.oSnr.wiener.mean,...
                performance.signalDistortion.dsd.wiener.overTime,...
                performance.signalDistortion.dsd.wiener.mean] ...
                = measurePerformance(performance,char(setup.filtStrings(iFiltStr)));
            case 'mvdr',
            % signal and noise powers (mvdr)   
            [performance.power.mvdr.sigPow,performance.power.mvdr.sigPowMean] = ...
                calculatePowers(data.mvdr.sigFrames,nBlockSkip);
            
            [performance.power.mvdr.noiPow,performance.power.mvdr.noiPowMean] = ...
                calculatePowers(data.mvdr.noiFrames,nBlockSkip);
            
            [performance.noiseReduction.oSnr.mvdr.overTime,...
                performance.noiseReduction.oSnr.mvdr.mean,...
                performance.signalDistortion.dsd.mvdr.overTime,...
                performance.signalDistortion.dsd.mvdr.mean] ...
                = measurePerformance(performance,char(setup.filtStrings(iFiltStr)));
            case 'trOff',
            % signal and noise powers (tr off)   
            for iRank = 1:size(data.trOff.sigFrames,3);
                [performance.power.trOff.sigPow(:,:,iRank),...
                    performance.power.trOff.sigPowMean(:,:,iRank)] = ...
                    calculatePowers(data.trOff.sigFrames(:,:,iRank),nBlockSkip);

                [performance.power.trOff.noiPow(:,:,iRank),...
                    performance.power.trOff.noiPowMean(:,:,iRank)] = ...
                    calculatePowers(data.trOff.noiFrames(:,:,iRank),nBlockSkip);

                [performance.noiseReduction.oSnr.trOff.overTime(:,:,iRank),...
                    performance.noiseReduction.oSnr.trOff.mean(:,:,iRank),...
                    performance.signalDistortion.dsd.trOff.overTime(:,:,iRank),...
                    performance.signalDistortion.dsd.trOff.mean(:,:,iRank)] ...
                    = measurePerformance(performance,char(setup.filtStrings(iFiltStr)),iRank);
            end
        end
    end
end

end

function [pow,powMean] = ...
    calculatePowers(data,nSkip)

[winLen,nFrames] = size(data);
for iFrames = 1:nFrames,
    pow(1,iFrames) = norm(data(:,iFrames))/winLen;
end

powMean = mean(pow(nSkip+1:end));
end

function [snrOverTime,snrMean,dsdOverTime,dsdMean] ...
    = measurePerformance(performance,filtStr,iRank)

if nargin < 3,
    iRank = 1;
end

snrOverTime = eval(['performance.power.',filtStr,'.sigPow(:,:,iRank)'])...
    ./eval(['performance.power.',filtStr,'.noiPow(:,:,iRank)']);

snrMean = eval(['performance.power.',filtStr,'.sigPowMean(:,:,iRank)'])...
    ./eval(['performance.power.',filtStr,'.noiPowMean(:,:,iRank)']);

dsdOverTime = performance.power.raw.sigPow...
    ./eval(['performance.power.',filtStr,'.sigPow(:,:,iRank)']);

dsdMean = performance.power.raw.sigPowMean...
    ./eval(['performance.power.',filtStr,'.sigPowMean(:,:,iRank)']);

end

