clc;clear all;close all;

addpath([cd,'\..\nonstationaryMultichanNoiseGenerator']);
addpath([cd,'\..\functions\']);

setup.sensorDistance = 0.1;
setup.speedOfSound = 343;
setup.nSensors = 3;
setup.noiseField = 'spherical';
setup.reverbTime = 0.2;

setup.sdnr = 0;
setup.ssnr = 30;

setup.nWin = 80;
setup.nFft = 2^nextpow2(setup.nWin);
setup.nFiltGrid = 1:6;
setup.minDis.signalRankMax = 6;
setup.forgetNoi = 0.99;
setup.forgetSig = 0.95;

setup.filtStrings = {'wiener','minDis','maxSnr'};

display(['Running script: ',mfilename]);
display(' ');

display('Enhancing...');
for idx = 1:length(setup.nFiltGrid),    
    setup.nFilt = setup.nFiltGrid(idx);
    setup.minDis.signalRanks = 1:min([setup.nFilt*setup.nSensors,setup.minDis.signalRankMax]);
    
    [signals,setup] = multichannelSignalGenerator(setup);

    [simulationData,setup] = stftEnhanceMultChanSignals(signals,setup);

    performance(idx,1) = stftMultichannelMeasurePerformance(simulationData,setup,1);
end
%%
display('Measuring performance...');
for idx = 1:length(setup.nFiltGrid),
    iSnrFbMean(1,idx) = performance(idx,1).noiseReduction.iSnr.fbMean;
    oSnrMaxSnrFbMean(1,idx) = performance(idx,1).noiseReduction.oSnr.maxSnr.fbMean;
    oSnrWienerFbMean(1,idx) = performance(idx,1).noiseReduction.oSnr.wiener.fbMean;
   
    dsdMaxSnrFbMean(1,idx) = performance(idx,1).signalDistortion.dsd.maxSnr.fbMean;
    dsdWienerFbMean(1,idx) = performance(idx,1).signalDistortion.dsd.wiener.fbMean;    
    
    for nn = 1:idx,
        oSnrMinDisFbMean(nn,idx) = (performance(idx,1).noiseReduction.oSnr.minDis.fbMean(nn));

        dsdMinDisFbMean(nn,idx) = (performance(idx,1).signalDistortion.dsd.minDis.fbMean(nn));
    end
end

%% save
% dateString = datestr(now,30);
% 
% save([mfilename,'_',dateString,'.mat']);

%% plots
close all;
figure(1);
plot(10*log10(mean(iSnrFbMean,3)),'k');
hold on;
plot(10*log10(mean(oSnrMaxSnrFbMean,3)));
plot(10*log10(mean(oSnrWienerFbMean,3)));
plot(10*log10(mean(oSnrMinDisFbMean,3).'),'g');
hold off;

figure(2);
plot(10*log10(mean(dsdMaxSnrFbMean,3)));
hold on;
plot(10*log10(mean(dsdWienerFbMean,3)));
plot(10*log10(mean(dsdMinDisFbMean,3).'),'g');
hold off;
