function [performance] = stftMultichannelMeasurePerformance(data,setup,flagFromSignals)

[nFreqs,nFrames] = size(data.raw.sigStft);
nWin = setup.nWin;
nBlockSkip = 5;
filtStrings = setup.filtStrings;


if flagFromSignals,
    % raw signal powers (channel 1)
    [performance.power.raw.sigPowNb,performance.power.raw.sigPowNbMean,...
        performance.power.raw.sigPowFb,performance.power.raw.sigPowFbMean] = ...
        calculatePowers(data.raw.sigStft(:,:,1),nFreqs,nWin,nBlockSkip);
   
    % raw noise powers (channel 1)
    [performance.power.raw.noiPowNb,performance.power.raw.noiPowNbMean,...
        performance.power.raw.noiPowFb,performance.power.raw.noiPowFbMean] = ...
        calculatePowers(data.raw.noiStft(:,:,1),nFreqs,nWin,nBlockSkip);
    
    [performance.noiseReduction.iSnr.nb,performance.noiseReduction.iSnr.fb,...
        performance.noiseReduction.iSnr.nbMean, ...
        performance.noiseReduction.iSnr.fbMean] ...
        = measurePerformance(performance,'raw');
    
    for iFiltStr=1:length(filtStrings),
        switch char(filtStrings(iFiltStr)),
            case 'maxSnr',
            % signal and noise powers (max snr)   
            [performance.power.maxSnr.sigPowNb,performance.power.maxSnr.sigPowNbMean,...
                performance.power.maxSnr.sigPowFb,performance.power.maxSnr.sigPowFbMean] = ...
                calculatePowers(data.maxSnr.sigStft,nFreqs,nWin,nBlockSkip);
            
            [performance.power.maxSnr.noiPowNb,performance.power.maxSnr.noiPowNbMean,...
                performance.power.maxSnr.noiPowFb,performance.power.maxSnr.noiPowFbMean] = ...
                calculatePowers(data.maxSnr.noiStft,nFreqs,nWin,nBlockSkip);
            
            [performance.noiseReduction.oSnr.maxSnr.nb,...
                performance.noiseReduction.oSnr.maxSnr.fb,...
                performance.noiseReduction.oSnr.maxSnr.nbMean,...
                performance.noiseReduction.oSnr.maxSnr.fbMean,...
                performance.signalDistortion.dsd.maxSnr.nb,...
                performance.signalDistortion.dsd.maxSnr.fb,...
                performance.signalDistortion.dsd.maxSnr.nbMean,...
                performance.signalDistortion.dsd.maxSnr.fbMean]...
                = measurePerformance(performance,char(filtStrings(iFiltStr)));
            
            case 'wiener',
            [performance.power.wiener.sigPowNb,performance.power.wiener.sigPowNbMean,...
                performance.power.wiener.sigPowFb,performance.power.wiener.sigPowFbMean] = ...
                calculatePowers(data.wiener.sigStft,nFreqs,nWin,nBlockSkip);
            
            [performance.power.wiener.noiPowNb,performance.power.wiener.noiPowNbMean,...
                performance.power.wiener.noiPowFb,performance.power.wiener.noiPowFbMean] = ...
                calculatePowers(data.wiener.noiStft,nFreqs,nWin,nBlockSkip);
            
            [performance.noiseReduction.oSnr.wiener.nb,...
                performance.noiseReduction.oSnr.wiener.fb,...
                performance.noiseReduction.oSnr.wiener.nbMean,...
                performance.noiseReduction.oSnr.wiener.fbMean,...
                performance.signalDistortion.dsd.wiener.nb,...
                performance.signalDistortion.dsd.wiener.fb,...
                performance.signalDistortion.dsd.wiener.nbMean,...
                performance.signalDistortion.dsd.wiener.fbMean]...
                = measurePerformance(performance,char(filtStrings(iFiltStr)));
            case 'minDis',
                for iRank = 1:size(data.minDis.sigStft,3),
                    [performance.power.minDis.sigPowNb(:,:,iRank),...
                        performance.power.minDis.sigPowNbMean(:,:,iRank),...
                        performance.power.minDis.sigPowFb(:,:,iRank),...
                        performance.power.minDis.sigPowFbMean(:,:,iRank)] = ...
                        calculatePowers(data.minDis.sigStft(:,:,iRank),nFreqs,nWin,nBlockSkip);

                    [performance.power.minDis.noiPowNb(:,:,iRank), ...
                        performance.power.minDis.noiPowNbMean(:,:,iRank), ...
                        performance.power.minDis.noiPowFb(:,:,iRank), ...
                        performance.power.minDis.noiPowFbMean(:,:,iRank)] = ...
                        calculatePowers(data.minDis.noiStft(:,:,iRank),nFreqs,nWin,nBlockSkip);

                    [performance.noiseReduction.oSnr.minDis.nb(:,:,iRank),...
                        performance.noiseReduction.oSnr.minDis.fb(:,:,iRank),...
                        performance.noiseReduction.oSnr.minDis.nbMean(:,:,iRank),...
                        performance.noiseReduction.oSnr.minDis.fbMean(:,:,iRank),...
                        performance.signalDistortion.dsd.minDis.nb(:,:,iRank),...
                        performance.signalDistortion.dsd.minDis.fb(:,:,iRank),...
                        performance.signalDistortion.dsd.minDis.nbMean(:,:,iRank),...
                        performance.signalDistortion.dsd.minDis.fbMean(:,:,iRank)]...
                        = measurePerformance(performance,char(filtStrings(iFiltStr)),iRank);
                end
            case 'trOff',
                for iRank = 1:size(data.trOff.sigStft,3),
                    [performance.power.trOff.sigPowNb(:,:,iRank),...
                        performance.power.trOff.sigPowNbMean(:,:,iRank),...
                        performance.power.trOff.sigPowFb(:,:,iRank),...
                        performance.power.trOff.sigPowFbMean(:,:,iRank)] = ...
                        calculatePowers(data.trOff.sigStft(:,:,iRank),nFreqs,nWin,nBlockSkip);

                    [performance.power.trOff.noiPowNb(:,:,iRank), ...
                        performance.power.trOff.noiPowNbMean(:,:,iRank), ...
                        performance.power.trOff.noiPowFb(:,:,iRank), ...
                        performance.power.trOff.noiPowFbMean(:,:,iRank)] = ...
                        calculatePowers(data.trOff.noiStft(:,:,iRank),nFreqs,nWin,nBlockSkip);

                    [performance.noiseReduction.oSnr.trOff.nb(:,:,iRank),...
                        performance.noiseReduction.oSnr.trOff.fb(:,:,iRank),...
                        performance.noiseReduction.oSnr.trOff.nbMean(:,:,iRank),...
                        performance.noiseReduction.oSnr.trOff.fbMean(:,:,iRank),...
                        performance.signalDistortion.dsd.trOff.nb(:,:,iRank),...
                        performance.signalDistortion.dsd.trOff.fb(:,:,iRank),...
                        performance.signalDistortion.dsd.trOff.nbMean(:,:,iRank),...
                        performance.signalDistortion.dsd.trOff.fbMean(:,:,iRank)]...
                        = measurePerformance(performance,char(filtStrings(iFiltStr)),iRank);
                end
        end
    end
end  

end

%%
function [powNb,powNbMean,powFb,powFbMean] = calculatePowers(stftData,fftLen,winLen,nSkip)

powNb = abs([stftData(:,:);conj(flipud(stftData(2:end-1,:)))]).^2/fftLen/winLen;
powNbMean = mean(powNb(:,nSkip+1:end),2);
powFb = sum(powNb,1);
powFbMean = mean(powFb(1,nSkip+1:end));
    
end

function [snrNb,snrFb,snrNbMean,snrFbMean,dsdNb,dsdFb,dsdNbMean,dsdFbMean] ...
    = measurePerformance(performance,filtStr,iRank)

if nargin < 3,
    iRank = 1;
end

snrNb = eval(['performance.power.',filtStr,'.sigPowNb(:,:,iRank)'])...
    ./eval(['performance.power.',filtStr,'.noiPowNb(:,:,iRank)']);
snrFb = eval(['performance.power.',filtStr,'.sigPowFb(:,:,iRank)'])...
    ./eval(['performance.power.',filtStr,'.noiPowFb(:,:,iRank)']);

snrNbMean = eval(['performance.power.',filtStr,'.sigPowNbMean(:,:,iRank)'])...
    ./eval(['performance.power.',filtStr,'.noiPowNbMean(:,:,iRank)']);
snrFbMean = eval(['performance.power.',filtStr,'.sigPowFbMean(:,:,iRank)'])...
    ./eval(['performance.power.',filtStr,'.noiPowFbMean(:,:,iRank)']);

dsdNb = performance.power.raw.sigPowNb...
    ./eval(['performance.power.',filtStr,'.sigPowNb(:,:,iRank)']);
dsdFb = performance.power.raw.sigPowFb...
    ./eval(['performance.power.',filtStr,'.sigPowFb(:,:,iRank)']);

dsdNbMean = performance.power.raw.sigPowNbMean...
    ./eval(['performance.power.',filtStr,'.sigPowNbMean(:,:,iRank)']);
dsdFbMean = performance.power.raw.sigPowFbMean...
    ./eval(['performance.power.',filtStr,'.sigPowFbMean(:,:,iRank)']);

end
